#!/bin/bash

_dropshell_completions() {
    local cur prev opts
    COMPREPLY=()
    cur="${COMP_WORDS[COMP_CWORD]}"
    prev="${COMP_WORDS[COMP_CWORD-1]}"
    
    # List of main commands
    opts="help version status servers templates autocomplete_list_servers autocomplete_list_services autocomplete_list_commands run install backup"

    # If we're completing the first argument, show all commands
    if [[ ${COMP_CWORD} -eq 1 ]] ; then
        COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
        return 0
    fi

    # Get available commands from dropshell
    local commands=($(dropshell autocomplete_list_commands))

    # Command-specific completions
    case "${prev}" in
        status|templates|autocomplete_list_servers|autocomplete_list_services|autocomplete_list_commands)
            # No additional completions for these commands
            COMPREPLY=()
            return 0
            ;;
        servers)
            # Show available server names for servers command
            local servers=($(dropshell autocomplete_list_servers))
            COMPREPLY=( $(compgen -W "${servers[*]}" -- ${cur}) )
            return 0
            ;;
        run|install|backup)
            # Handle completion for run/install/backup commands
            if [[ ${COMP_CWORD} -eq 2 ]]; then
                # Second argument is server name
                local servers=($(dropshell autocomplete_list_servers))
                COMPREPLY=( $(compgen -W "${servers[*]}" -- ${cur}) )
                return 0
            elif [[ ${COMP_CWORD} -eq 3 ]]; then
                # Third argument is service name
                local server_name="${COMP_WORDS[2]}"
                local services=($(dropshell autocomplete_list_services "$server_name"))
                COMPREPLY=( $(compgen -W "${services[*]}" -- ${cur}) )
                return 0
            elif [[ ${COMP_CWORD} -eq 4 && "${COMP_WORDS[1]}" == "run" ]]; then
                # Fourth argument is command name - only for run command
                COMPREPLY=( $(compgen -W "${commands[*]}" -- ${cur}) )
                return 0
            fi
            ;;
        *)
            # if ${prev} is not in ${commands}
            if [[ ! " ${commands[*]} " =~ " ${prev} " ]]; then
                COMPREPLY=()
                return 0
            fi
            # Handle completion for template commands
            if [[ ${COMP_CWORD} -eq 2 ]]; then
                # Second argument is server name
                local servers=($(dropshell autocomplete_list_servers))
                COMPREPLY=( $(compgen -W "${servers[*]}" -- ${cur}) )
                return 0
            elif [[ ${COMP_CWORD} -eq 3 ]]; then
                # Third argument is service name
                local server_name="${COMP_WORDS[2]}"
                local services=($(dropshell autocomplete_list_services "$server_name"))
                COMPREPLY=( $(compgen -W "${services[*]}" -- ${cur}) )
                return 0
            fi
            ;;
    esac
}

# Register the completion function
complete -F _dropshell_completions dropshell 